/*
	binop.h
	<c> Risto Holopainen 2021
	
	collection of binary operations on signals

	This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef BINOPS_H
#define BINOPS_H

#include <cmath>

namespace binopsic
{

class lp
{
	public:
		lp();
		double filter(double x);

	private:
		double x1, x2;
		double y;
};

lp::lp()
{
	x1 = x2 = 0;
	y = 0;
}

double lp::filter(double x0)
{
	y = .2 * (x0 + 2*x1 + x2 + y);
	x2 = x1;
	x1 = x0;
	return y;
}


double mix(double x, double y)
{
	return .5 * (x + y);
}

double mult(double x, double y)
{
	return x*y;
}

double nl_mix(double x, double y)
{
	return tanh(x + y);
}

double min(double x, double y)
{
	if(x < y)
		return x;
	else
		return y;
}


double max(double x, double y)
{
	if(x > y)
		return x;
	else
		return y;
}



enum binops {Mix, Mult, NLmix, Min, Max};
typedef double (*fp) (double, double);


class bin_op
{
	public:
		bin_op(binops BOP);
		double process(double u, double v);
		void method(binops bop);
		void next_method();

	private:
		lp LP1, LP2;
		double x, y;
		fp howto;
		fp toolbox[5];
		int k;

};

bin_op::bin_op(binops BOP)
{
	toolbox[0] = mix;
	toolbox[1] = mult;
	toolbox[2] = nl_mix;
	toolbox[3] = min;
	toolbox[4] = max;
	k = 0;

	this->method(BOP);
}


double bin_op::process(double u, double v)
{
	x = LP1.filter(u);
	y = LP2.filter(v);
	return howto(x, y);
}


void bin_op::method(binops bop)
{
	switch (bop)
		{
		case Mix :
			{howto = mix; break;}
		case Mult :
			{howto = mult; break;}
		case NLmix :
			{howto = nl_mix; break;}
		case Min :
			{howto = min; break;}
		case Max :
			{howto = max; break;}
		}
}

void bin_op::next_method()
{
	if(++k >= 5)
		k = 0;
	
	howto = toolbox[k];
}

}
#endif

