
/*
	line.h 
	simple sound synthesis
	Copyright Risto Holopainen 2021

	generates waveforms from line segments
	
	This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/


#ifndef LINESYNTH_H
#define LINESYNTH_H

class line
{
	public:
		line();
		void set(double slope, double lo, double hi);
		void set(const double params[3]);
		void draw(double *buf, unsigned int L);		// generate L audio samples
		double draw();								// generate 1 audio sample
		
	private:
		int dir;
		double slope;
		int length;
		double x;
		double bound_hi, bound_lo;
};

line::line()
{
	length = 10;
	slope = .25;
	dir = 1;	// or -1
	x = 0;
	bound_hi = 1;
	bound_lo = -1;
}


void line::draw(double *buf, unsigned int L)
{
	for(unsigned int i=0; i<L; i++)
		{
		x += dir*slope;
		if(x >= bound_hi || x <= bound_lo)
			{
			dir *= -1;
			x = (x >= bound_hi) ? bound_hi : bound_lo;
			}
		buf[i] = x;
		}
}

double line::draw()
{
	x += dir*slope;
	if(x >= bound_hi || x <= bound_lo)
		{
		dir *= -1;
		x = (x >= bound_hi) ? bound_hi : bound_lo;
		}

	return x;
}


void line::set(double slp, double lo, double hi)
{
	slope = slp;
	if(lo < hi && lo >= -1 && hi <= +1)
		{
		bound_lo = lo;
		bound_hi = hi;
		}
}

void line::set(const double p[3])
{
	slope = p[0];
	if(p[1] < p[2] && p[1] >= -1 && p[2] <= +1)
		{
		bound_lo = p[1];
		bound_hi = p[2];
		}
}


#endif		// def linesynth
