/*
	diff
	<c> Risto Holopainen 2010
	differentiating filter (FIR, odd order)
	
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
*/


#ifndef DIFF_H
#define DIFF_H

#include <cassert>

class diff
{
	public:
		diff(int sz);
		~diff();
		double filter(double);

	private:
		int N;
		int n;
		double *h;
		double *b;
		double norm;
};


diff::diff(int sz)
{
	assert (sz & 1);	// udda
	N = sz;
	h = new double[N/2];
	b = new double[2*N];
	n = 0;
	double sum = 0;

	for(int i=1; i<N/2; i++)
		{
		h[i] = 1.0/i;
		sum += h[i];
		}
	norm = 1.0/sum;
	for(int i=0; i<2*N; i++)
		b[i] = 0;
}

diff::~diff()
{
	delete h;
	delete b;
}


double diff::filter(double x)
{
	double y = 0;
	int c = n + N/2;

	for(int i=1; i<N/2; i++)
		{
		y += h[i] * (b[c+i] - b[c-i]);
		}
	
	if(++n >= N)
		n = 0;

	b[n] = b[N+n] = x;
	return norm*y;
}

#endif

